/*
 * deathLine.c
 *
 * Created: 17.10.2017 11:24:23
 *  Author: dsosnowski
 */ 


#define F_CPU 8000000UL
#include <avr/io.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "wdog.h"
#include "buttons.h"
#include "ledLine.h"
#include "ee_mem.h"
#include "eff.h"


#define MAIN_DELAY		3


#define PARAM_EFFECT	0
#define PARAM_SPEED		1
#define PARAM_TRACE		2
#define PARAM_PERIOD	3

#define MAX_DELAY		40
#define MIN_DELAY		1
#define DEFAULT_DELAY	30

#define MAX_TRACE		50
#define MIN_TRACE		1
#define DEFAULT_TRACE	25

#define MAX_PERIOD		50
#define MIN_PERIOD		1
#define DEFAULT_PERIOD	25

#define MAX_EFFECT		7
#define MIN_EFFECT		0
#define DEFAULT_EFFECT	0

#define SAVE_PARAM_DELAY	600	
#define EEMEM_ADR			8
#define EEMEM_STORE_LEN		5
#define DEFAULT_MAGIC_VAL	55	


//------------------------------------------------------------------
void MainHardInit(void);
void SystemDefault(void);
void SystemInfo(uint8_t pos);
void SystemParamChange(uint8_t param);

struct {
	uint8_t number;
	uint8_t delay;
	uint8_t trace;
	uint8_t period;
	uint8_t def;
} effect;

struct {
	uint8_t param;
	uint16_t saveParam;
} system;


//------------------------------------------------------------------
int main(void)
{
	uint8_t tim = 0;
	
	WdtSet(7);
	MainHardInit();
	
	SettRestore(&effect.number, EEMEM_ADR, EEMEM_STORE_LEN);
	if (effect.def != DEFAULT_MAGIC_VAL){
		SystemDefault();
	}

		if (effect.delay > MAX_DELAY){
			effect.delay = 0;
		}
		if (effect.trace > MAX_TRACE){
			effect.trace = 0;
		}
		if (effect.number > MAX_EFFECT){
			effect.number = MAX_EFFECT;
		}		
	
	LineTraceSpeed(effect.trace);
	LineInit();

	while(1){		
		//-----------------------------------
		ButtonProc();
		if (ButtonActionGet() > 0){
			if (ButtonActionGet() == BUTTON1_CLICK){
				SystemParamChange(PARAM_EFFECT);
				system.saveParam = SAVE_PARAM_DELAY;
				SystemInfo(effect.number);
				
			}
			if (ButtonActionGet() == BUTTON2_CLICK){
				SystemParamChange(PARAM_SPEED);
				system.saveParam = SAVE_PARAM_DELAY;
			}
			if (ButtonActionGet() == BUTTON3_CLICK){
				SystemParamChange(PARAM_TRACE);
				system.saveParam = SAVE_PARAM_DELAY;
			}
		}
		
		//-----------------------------------
		tim++;
		if (tim > effect.delay){
			tim = 0;
			switch (effect.number){
				case 0: NightRider(); break;
				case 1: Sopel(); break;
				case 2: SopelBack(); break;
				case 3: NightRider2(); break;
				case 4: SopelSlow(); break;
				case 5: SopelBackSlow(); break;
				case 6: RandomPoints(); break;
				case 7: RandomSopel(); break;
			}
		}
		
		//-----------------------------------
		if (system.saveParam > 0){
			system.saveParam--;
			if (system.saveParam == 1){
				effect.def = DEFAULT_MAGIC_VAL;
				cli();
				SettStore(&(effect.number), EEMEM_ADR, EEMEM_STORE_LEN);
				sei();
			}
		}
		
		//-----------------------------------
		_delay_ms(MAIN_DELAY);
		WdtReset();
    }
}

//----------------------------------------------------------
void MainHardInit(void){
	ACSR = (1<<ACD);
	PORTB = 0xFF;
	DDRB = 0xFE;
	PORTD = 0xFF;
	DDRD = 0x6B;
}

//----------------------------------------------------------
void SystemDefault(void){
	effect.number = DEFAULT_EFFECT;
	effect.delay = DEFAULT_DELAY;
	effect.trace = DEFAULT_TRACE;
	effect.period = DEFAULT_PERIOD;
}

//----------------------------------------------------------
void SystemInfo(uint8_t pos){
	uint8_t mask = ledMask[pos];
	cli();
		PORTB = 0xFF;
		PORTD = 0xFF;
		_delay_ms(200);
		PORTB = ~(mask);
		PORTD = 0xFF;
		_delay_ms(300);
		PORTB = 0xFF;
		PORTD = 0xFF;
		_delay_ms(200);
	sei();
}

//----------------------------------------------------------
void SystemParamChange(uint8_t param){
	static int8_t speedDir;
	static int8_t traceDir;

	switch (param){
		case PARAM_SPEED:{
			if (speedDir < 0) {
				effect.delay -= 1;
			} else {
				effect.delay += 1;
			}
			
			if (effect.delay == MAX_DELAY){
				speedDir = -1;
			}
			if (effect.delay > MAX_DELAY){
				effect.delay = 0;
			}
			if (effect.delay < MIN_DELAY){
				effect.delay = MIN_DELAY;
				speedDir = 1;
				
			}
		}break;
		case PARAM_TRACE:{
			if (traceDir < 0) {
				effect.trace -= 1;
			} else {
				effect.trace += 1;
			}
			
			if (effect.trace == MAX_TRACE){
				traceDir = -1;
			}
			if (effect.trace > MAX_TRACE){
				effect.trace = 0;
			}
			if (effect.trace < MIN_TRACE){
				effect.trace = MIN_TRACE;
				traceDir = 1;
			}
			LineTraceSpeed(effect.trace);		
		}break;
		case PARAM_EFFECT:{
			effect.number += 1;
			if (effect.number == (MAX_EFFECT+1)){
				//effect.number = MAX_EFFECT;
				effect.number = MIN_EFFECT;
			}
			if (effect.number > MAX_EFFECT){
				effect.number = MAX_EFFECT;
			}
			if (effect.number < MIN_EFFECT){
				effect.number = MIN_EFFECT;
			}
		}break;
	}
}