/*
 * ledLine.c
 *
 * Created: 17.10.2017 10:20:26
 *  Author: dsosnowski
 */ 

#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include "ledLine.h"



#define INTENSIVITY_PERIOD	100
#define TRACE_DEFAULT_PERIOD	150
#define TRACE_THRESHOLD		70

#define TIMER_START			TCCR0B = (1<<CS01 | 1<<WGM02)
#define TIMER_CONFIG		TCCR0A = (1<<WGM01 | 1<<WGM00)
#define TIMER_STOP			TCCR0B = 0
#define TIMER_RESET			TCNT0 = 0
#define TIMER_ENABLE		TIMSK |= (1<<TOIE0)
#define TIMER_DISABLE		TIMSK &=~(1<<TOIE0)
#define TIMER_CLEAR			TIFR  |= (1<<TOV0)
#define TIMER_VALUE			TCNT0
#define TIMER_PERIOD		OCR0A
/*#define TIMER_ENABLE		TIMSK |= (1<<OCIE0A)
#define TIMER_DISABLE		TIMSK &=~(1<<OCIE0A)
#define TIMER_CLEAR			TIFR  |= (1<<OCF0A)
*/


const uint8_t ledMask[LINE_LEN] = {LED1_PIN, LED2_PIN, LED3_PIN, LED4_PIN, 
					 LED5_PIN, LED6_PIN, LED7_PIN, LED8_PIN, 
					 LED9_PIN, LED10_PIN, LED11_PIN, LED12_PIN};

struct {
	volatile uint8_t Intensivity[LINE_LEN];
	uint8_t lead;
	uint8_t trace;
} line;
	 


//----------------------------------------------------------
void LineInit(void){
	PORTB = 0xFF;
	PORTD = 0xFF;
	DDRB = 0xFF;
	DDRD = 0x78;
	for(uint8_t i = 0; i < LINE_LEN; i++){
		LineDotSet(i);
	}
	TIMER_STOP;
	TIMER_PERIOD = 100;
	TIMER_CONFIG;
	TIMER_RESET;
	TIMER_CLEAR;
	TIMER_ENABLE;
	TIMER_START;
	sei();
}

//----------------------------------------------------------
void LineState(uint8_t reference){
	uint8_t outb = 0;
	uint8_t outd = 0;
	static uint8_t ledTrace;

	ledTrace++;
	
	for(uint8_t i = 0; i < LINE_LEN; i++){
		if (line.Intensivity[i] > reference){
			if (i < 8){
				outb |= ledMask[i];
			} else {
				outd |= ledMask[i];				
			}
		}
		
		if (ledTrace >= line.trace){
			if (i != line.lead){
				if (line.Intensivity[i] > TRACE_THRESHOLD) line.Intensivity[i] = TRACE_THRESHOLD;
				if (line.Intensivity[i] > 0)line.Intensivity[i]--;
			}
		}
	}
	if (ledTrace >= line.trace) {
		ledTrace = 0;
	}

	outb = ~(outb);
	outd = ~(outd);
	PORTB = outb;
	PORTD = outd;
}

//----------------------------------------------------------
SIGNAL(TIMER0_OVF_vect){
	static uint8_t mainCount;
	mainCount++;
	if (mainCount > INTENSIVITY_PERIOD){
		mainCount = 0;
	}
	LineState(mainCount);
}

//----------------------------------------------------------
void LineDotSet(uint8_t dot){
	if (dot < LINE_LEN){
		line.lead = dot;
		line.Intensivity[dot] = (INTENSIVITY_PERIOD - 5);
	} else {
		line.lead = LINE_LEN;
	}
}

//----------------------------------------------------------
void LineTraceSpeed(uint8_t speed){
	line.trace = ((speed * 5) + 1);
}
